# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - EventAggregatorSummary Action

    @copyright: 2008, 2009, 2010 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2003-2008 MoinMoin:ThomasWaldmann,
                2004-2006 MoinMoin:AlexanderSchremmer,
                2007 MoinMoin:ReimarBauer.
                2009 Cristian Rigamonti <rigamonti@fsfeurope.org>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin import config
from MoinMoin.Page import Page
import MoinMoin.util # for MoinMoin 1.5.x
from MoinMoin import wikiutil
import EventAggregatorSupport

Dependencies = ['pages']

# Action class and supporting functions.

class EventAggregatorSummary(ActionBase):

    "A summary dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request

        category_list = []

        for category_name, category_pagename in \
            EventAggregatorSupport.getCategoryMapping(
                EventAggregatorSupport.getCategories(request),
                request):

            category_list.append('<option value="%s">%s</option>' % (category_pagename, category_name))

        month_list = []
        month_list.append('<option value=""></option>')

        for month in range(1, 13):
            month_label = _(EventAggregatorSupport.getMonthLabel(month))
            month_list.append('<option value="%02d">%s</option>' % (month, month_label))

        descriptions_list = [
            '<option value="%s">%s</option>' % ("page", _("page")),
            '<option value="%s">%s</option>' % ("comment", _("comment"))
            ]

        format_list = [
            '<option value="%s">%s</option>' % ("iCalendar", _("iCalendar")),
            '<option value="%s">%s</option>' % ("RSS", _("RSS 2.0"))
            ]

        d = {
            "buttons_html" : buttons_html,
            "category_label" : _("Categories"),
            "category_list" : "\n".join(category_list),
            "month_list" : "\n".join(month_list),
            "start_label" : _("Start year and month"),
            "start_year_default" : "",
            "end_label" : _("End year and month"),
            "end_year_default" : "",
            "descriptions_label" : _("Use descriptions from..."),
            "descriptions_list" : "\n".join(descriptions_list),
            "format_label" : _("Summary format"),
            "format_list" : "\n".join(format_list),
            }

        return '''
<table>
    <tr>
        <td class="label"><label>%(category_label)s</label></td>
        <td class="content">
            <select multiple="multiple" name="category">
                %(category_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(start_label)s</label></td>
        <td>
            <select name="start-month">
                %(month_list)s
            </select>
            <input name="start-year" type="text" value="%(start_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(end_label)s</label></td>
        <td>
            <select name="end-month">
                %(month_list)s
            </select>
            <input name="end-year" type="text" value="%(end_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(descriptions_label)s</label></td>
        <td class="content">
            <select name="descriptions">
                %(descriptions_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(format_label)s</label></td>
        <td class="content">
            <select name="format">
                %(format_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the iCalendar resource."

        _ = self._
        form = self.request.form

        # If no category names exist in the request, an error message is
        # returned.

        category_names = form.get("category", [])

        if not category_names:
            return 0, _("No categories specified.")

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

def getQuotedText(text):

    "Return the 'text' quoted for iCalendar purposes."

    return text.replace(";", r"\;").replace(",", r"\,")

def write_resource(request):

    """
    For the given 'request', write an iCalendar summary of the event data found
    in the categories specified via the "category" request parameter, using the
    "start" and "end" parameters (if specified). Multiple "category" parameters
    can be specified.
    """

    form = request.form

    category_names = form.get("category", [])
    format = form.get("format", ["iCalendar"])[0]
    descriptions = form.get("descriptions", ["page"])[0]

    # Otherwise, produce an iCalendar resource.

    calendar_start = EventAggregatorSupport.getFormMonth(request, None, "start")
    calendar_end = EventAggregatorSupport.getFormMonth(request, None, "end")

    # Look for separate start and end years and months.

    if calendar_start is None:
        calendar_start = EventAggregatorSupport.getFormMonthPair(request, "start-year", "start-month")

    if calendar_end is None:
        calendar_end = EventAggregatorSupport.getFormMonthPair(request, "end-year", "end-month")

    events, shown_events, all_shown_events, earliest, latest = \
        EventAggregatorSupport.getEvents(request, category_names, calendar_start, calendar_end)

    latest_timestamp = EventAggregatorSupport.setEventTimestamps(request, all_shown_events)

    # Output summary data...

    if EventAggregatorSupport.isMoin15():
        send_headers = request.http_headers
    else:
        send_headers = request.emit_http_headers

    # Define headers.

    if format == "iCalendar":
        headers = ["Content-Type: text/calendar; charset=%s" % config.charset]
    elif format == "RSS":
        headers = ["Content-Type: application/rss+xml; charset=%s" % config.charset]

    # Define the last modified time.

    if latest_timestamp is not None:
        headers.append("Last-Modified: %s" % EventAggregatorSupport.getHTTPTimeString(latest_timestamp))

    send_headers(headers)

    # iCalendar output...

    if format == "iCalendar":
        request.write("BEGIN:VCALENDAR\r\n")
        request.write("PRODID:-//MoinMoin//EventAggregatorSummary\r\n")
        request.write("VERSION:2.0\r\n")

        for event_page in all_shown_events:
            event_details = event_page.getEventDetails()

            # Get the summary details.

            event_summary = event_page.getEventSummary()
            link = event_page.getPageURL(request)

            # Output the event details.

            request.write("BEGIN:VEVENT\r\n")
            request.write("UID:%s\r\n" % link)
            request.write("URL:%s\r\n" % link)
            request.write("DTSTAMP:%04d%02d%02dT%02d%02d%02dZ\r\n" % event_details["created"][:6])
            request.write("LAST-MODIFIED:%04d%02d%02dT%02d%02d%02dZ\r\n" % event_details["last-modified"][:6])
            request.write("SEQUENCE:%d\r\n" % event_details["sequence"])
            request.write("DTSTART;VALUE=DATE:%04d%02d%02d\r\n" % event_details["start"].as_tuple())
            request.write("DTEND;VALUE=DATE:%04d%02d%02d\r\n" % event_details["end"].next_day().as_tuple())
            request.write("SUMMARY:%s\r\n" % getQuotedText(event_summary))

            # Optional details.

            if event_details.has_key("topics") or event_details.has_key("categories"):
                request.write("CATEGORIES:%s\r\n" % ",".join(
                    [getQuotedText(topic) for topic in event_details.get("topics") or event_details.get("categories")]
                    ))
            if event_details.has_key("location"):
                request.write("LOCATION:%s\r\n" % getQuotedText(event_details["location"]))

            request.write("END:VEVENT\r\n")

        request.write("END:VCALENDAR\r\n")

    elif format == "RSS":

        # Using the page name and the page URL in the title, link and
        # description.

        request.write('<rss version="2.0">\r\n')
        request.write('<channel>\r\n')
        request.write('<title>%s</title>\r\n' % request.getPathinfo()[1:])
        request.write('<link>%s%s</link>\r\n' % (request.getBaseURL(), request.getPathinfo()))
        request.write('<description>Events published on %s%s</description>\r\n' % (request.getBaseURL(), request.getPathinfo()))
        request.write('<lastBuildDate>%s</lastBuildDate>\r\n' % EventAggregatorSupport.getHTTPTimeString(latest_timestamp))
 
        # Sort all_shown_events by start date, reversed:
        #
        #  * event_details are dictionaries, with the "start" entry providing
        #    the start date
        #
        # So we use as sorting key the "start" key from the event details.

        ordered_events = EventAggregatorSupport.getOrderedEvents(all_shown_events)
        ordered_events.reverse()

        for event_page in ordered_events:
            event_details = event_page.getEventDetails()

            # Get the summary details.

            event_summary = event_page.getEventSummary()
            link = event_page.getPageURL(request)

            request.write('<item>\r\n')
            request.write('<title>%s</title>\r\n' % wikiutil.escape(event_summary))
            request.write('<link>%s</link>\r\n' % link)

            # Write a description according to the preferred source of
            # descriptions.

            if descriptions == "page":
                description = event_details.get("description", "")
            else:
                description = event_details["last-comment"]

            request.write('<description>%s</description>\r\n' % wikiutil.escape(description))

            for topic in event_details.get("topics") or event_details.get("categories") or []:
                request.write('<category>%s</category>\r\n' % topic)

            request.write('<pubDate>%s</pubDate>\r\n' % EventAggregatorSupport.getHTTPTimeString(event_details["created"]))
            request.write('<guid>%s#%s</guid>\r\n' % (link, event_details["sequence"]))
            request.write('</item>\r\n')

        request.write('</channel>\r\n')
        request.write('</rss>\r\n')

    if EventAggregatorSupport.isMoin15():
        raise MoinMoin.util.MoinMoinNoFooter

# Action function.

def execute(pagename, request):
    EventAggregatorSummary(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
